/*
  +----------------------------------------------------------------------+
  | Portable System Library                                              |
  | Author: Sukchan Lee (acetcom@postech.ac.kr)                          |
  +----------------------------------------------------------------------+

  $Id: fileio.h 113 2004-04-06 06:02:40Z acetcom $
*/

#ifndef __PSYS_FILEIO_H__
#define __PSYS_FILEIO_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "psys/common.h"
#include "psys/memory.h"

/**
 * @defgroup fileio File Input/Output
 * @ingroup libpsys 
 *
 * @{
 */

/** Open the file for reading */
#define PSYS_READ       1           
/** Open the file for writing */
#define PSYS_WRITE      2           
/** Create the file if not there */
#define PSYS_CREATE     4           
/** Append to the end of the file */
#define PSYS_APPEND     8           
/** Open the file and truncate to 0 length */
#define PSYS_TRUNCATE   16          
/** Open the file in binary mode */
#define PSYS_BINARY     32          
/** Open should fail if PSYS_CREATE and file exists. */
#define PSYS_EXCL       64          

#define PSYS_SEEK_SET   SEEK_SET
#define PSYS_SEEK_CUR   SEEK_CUR
#define PSYS_SEEK_END   SEEK_END

#define APR_UREAD       0x0400 /** Read by user */
#define APR_UWRITE      0x0200 /** Write by user */
#define APR_UEXECUTE    0x0100 /** Execute by user */

#define APR_GREAD       0x0040 /** Read by group */
#define APR_GWRITE      0x0020 /** Write by group */
#define APR_GEXECUTE    0x0010 /** Execute by group */

#define APR_WREAD       0x0004 /** Read by others */
#define APR_WWRITE      0x0002 /** Write by others */
#define APR_WEXECUTE    0x0001 /** Execute by others */

/** OS ϴ Default Permission ǹѴ. */
#define PSYS_DEFAULT_PERM   0x0FFF 

typedef p_uint32_t          psys_fileattrs_t;
typedef p_uint32_t          psys_fileperms_t;
typedef struct psys_file_t  psys_file_t;

/**
 *  .
 * @param new_file  
 * @param fname  ̸ ( θ Ѵ.)
 * @param flag    Flag̴.(OR'ed   Ѵ.)
 * <PRE>
 *         PSYS_READ
 *         PSYS_WRITE
 *         PSYS_CREATE
 *         PSYS_APPEND
 *         PSYS_TRUNCATE
 *         PSYS_BINARY
 *         PSYS_EXCL
 * </PRE>
 * @param perm    Ѵ.
 * <PRE> 
 *         PSYS_UREAD
 *         PSYS_UWRITE
 *         PSYS_UEXECUTE
 *
 *         PSYS_GREAD
 *         PSYS_GWRITE
 *         PSYS_GEXECUTE
 *
 *         PSYS_WREAD
 *         PSYS_WWRITE
 *         PSYS_WEXECUTE
 * </PRE>
 * @param p psys_file_t  ޸
 * @return  
 * @remark  perm PSYS_DEFAULT_PERM ϸ OS ϴ  
 *  Ѵ.
 */
psys_status_t psys_fopen(psys_file_t **new_file, const char *fname, 
        p_int32_t flag, psys_fileperms_t perm, psys_mem_pool_t *p);

/**
 *  ݴ´.
 * @param file  
 * @return ϸ 0 ְ ׷   error code ش.
 */
psys_status_t psys_fclose(psys_file_t *file);

/**
 *   ˻Ѵ.
 * @param file  
 */
psys_status_t psys_feof(psys_file_t *file);

/**
 * Ͽ ͸ д´.
 * @param file  
 * @param buf  ͸ ϴ 
 * @param nbytes   
 */
psys_status_t psys_fread(psys_file_t *file, void *buf, size_t *nbytes);

/**
 * Ͽ ͸ .
 * @param file  
 * @param buf Ͽ  
 * @param nbytes   
 */
psys_status_t psys_fwrite(psys_file_t *file, const void *buf, size_t *nbytes);

/**
 * Ͽ  ϳ д´.
 * @param ch о ڸ ϴ 
 * @param file  
 */
psys_status_t psys_fgetc(char *ch, psys_file_t *file);

/**
 * Ͽ  ϳ .
 * @param ch Ͽ   
 * @param file  
 */
psys_status_t psys_fputc(char ch, psys_file_t *file);

/**
 * Ͽ ڿ д´.
 * @param str о ڿ ϴ 
 * @param len ڿ 
 * @param file  
 */
psys_status_t psys_fgets(char *str, int len, psys_file_t *file);

/**
 * Ͽ ڿ .
 * @param str Ͽ  ڿ
 * @param file  
 */
psys_status_t psys_fputs(const char *str, psys_file_t *file);

/**
 *  ۸ FlushѴ.
 * @param  
 */
psys_status_t psys_fflush(psys_file_t *file);

/**
 *  а   ġ ͸ ̵Ų.
 * @param file  
 * @param where  ġ ͸ ̵Ű  
 * <PRE>
 *        PSYS_SEEK_SET - ó offsetŭ 
 *        PSYS_SEEK_CUR -  ġ offsetŭ 
 *        PSYS_SEEK_END -   offsetŭ
 * </PRE>
 * @param offset  ̵ų ũ
 * @remark offset  ̵Ų offset returnѴ.
 */
psys_status_t psys_fseek(psys_file_t *file, int where, off_t *offset);

/** @} */

#ifdef __cplusplus
}
#endif

#endif
