/*
  +----------------------------------------------------------------------+
  | Portable System Library                                              |
  | Author: Sukchan Lee (acetcom@postech.ac.kr)                          |
  +----------------------------------------------------------------------+

  $Id: string.h 113 2004-04-06 06:02:40Z acetcom $
*/

#ifndef __PSYS_STRING_H__
#define __PSYS_STRING_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "psys/common.h"
#include "psys/memory.h"

/**
 * @defgroup string String Functions
 * @ingroup libpsys 
 *
 * @{
 */
#if !HAVE_MEMCHR
void *memchr(const void *s, int c, size_t n);
#endif

#if !HAVE_STRSTR
char *strstr(char *s1, char *s2);
#endif

#if !HAVE_STRLCPY
size_t strlcpy(char *dst, const char *src, size_t n);
#endif

#if !HAVE_STRLCAT
size_t strlcat(char *dst, const char *src, size_t n);
#endif

#if (!HAVE_STRCASECMP) && (HAVE_STRICMP) 
#define strcasecmp(s1, s2) stricmp(s1, s2)
#elif (!HAVE_STRCASECMP && !HAVE_STRICMP)
int strcasecmp(const char *s1, const char *s2);
#endif

#if (!HAVE_STRNCASECMP) && (HAVE_STRNICMP)
#define strncasecmp(s1, s2, n) strnicmp(s1, s2, n)
#elif (!HAVE_STRNCASECMP && !HAVE_STRNICMP)
int strncasecmp(const char *s1, const char *s2, size_t n);
#endif

#if !HAVE_VSNPRINTF
int vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap);
#endif

#if !HAVE_SNPRINTF
int snprintf(char *str, size_t str_m, const char *fmt, ...);
#endif

/**
 * Memory Pool ڿ Ҵϰ Ͽ ش.
 *
 * @param p Memory Pool
 * @param s  ڿ
 * @return ο ڿ
 *
 * @remark ޸  psys_pool_destroy() ̿Ѵ.
 */
char *psys_strdup(psys_mem_pool_t *p, const char *s);

/**
 * Memory Pool ũⰡ n ڸ  ڿ Ҵϰ Ѵ.
 *
 * @param p Memory Pool 
 * @param s  ڿ
 * @param n ڿ ũ
 * @return ο ڿ
 *
 * @remark ޸  psys_pool_destroy() ̿Ѵ.
 */
char *psys_strndup(psys_mem_pool_t *p, const char *s, size_t n);

/**
 * Memory Pool ũⰡ n ޸𸮸 Ҵϰ Ѵ.
 *
 * @param p Memory Pool 
 * @param m  ޸
 * @param n ޸ ũ
 * @return ο ޸
 *
 * @remark ޸  psys_pool_destroy() ̿Ѵ.
 */
void *psys_memdup(psys_mem_pool_t *p, const void *m, size_t n);

/**
 * Memory Pool int type ڿ  ڿ Ѵ.
 *
 * @param p Memory Pool 
 * @param n int type 
 * @return ڿ
 */
char *psys_itoa(psys_mem_pool_t *p, int n);

/**
 * Memory Pool long type ڿ  ڿ Ѵ.
 *
 * @param p Memory Pool 
 * @param n long type 
 * @return ڿ
 */
char *psys_ltoa(psys_mem_pool_t *p, long n);

/**
 * ڿ 64-bits type ڷ ȯѴ.
 *
 * @param buf ڷ ٲ ڿ
 * @param end ڷ ٲ  ִ   Ѵ.  NULL 
 *        ƴϸ, ڿ ߿ ڷ ٲ   ڰ Ѵٴ ̴.
 * @param base
 *      ڿ base ̷ ִٴ   ǥѴ.
 *      base  2 36̰ 0    ִ.
 *       base 0̸ ڿ 10 ϰ, 
 *      base 0ε, ڿ "0x" ϸ 16 Ѵ.
 * @return 64-bits type 
 */
p_int64_t psys_strtoi64(const char *buf, char **end, int base);

/**
 * 10 ڿ 64-bits type ڷ ȯѴ.
 *
 * @param buf ڷ ٲ ڿ
 * @return 64-bits type 
 */
p_int64_t psys_atoi64(const char *buf);

/** @} */

#ifdef __cplusplus
}
#endif

#endif
