/*
  +----------------------------------------------------------------------+
  | Portable System Library                                              |
  | Author: Sukchan Lee (acetcom@postech.ac.kr)                          |
  +----------------------------------------------------------------------+

  $Id: memory.c 102 2004-03-26 15:36:02Z acetcom $
*/

#include "psys/common.h"
#include "psys/errno.h"
#include "psys/memory.h"

#include "CuTest.h"

#define ALLOC_BYTES 1024

static psys_mem_pool_t *pmain = NULL;
static psys_mem_pool_t *pchild = NULL;

static void psys_mem_pool_create_parent_test(CuTest *tc)
{
    p_uint32_t rv;

    rv = psys_mem_pool_create(&pmain, NULL);
    CuAssertIntEquals(tc, rv, PSYS_SUCCESS);
    CuAssertPtrNotNull(tc, pmain);
}

static void psys_mem_pool_create_child_test(CuTest *tc)
{
    p_uint32_t rv;

    rv = psys_mem_pool_create(&pchild, pmain);
    CuAssertIntEquals(tc, rv, PSYS_SUCCESS);
    CuAssertPtrNotNull(tc, pchild);
}

static void psys_malloc_test(CuTest *tc)
{
    int i;
    char *alloc;
    
    alloc = psys_malloc(pmain, ALLOC_BYTES);
    CuAssertPtrNotNull(tc, alloc);

    for (i = 0; i < ALLOC_BYTES; i++) 
    {
        char *ptr = alloc + i;
        *ptr = 0xa;
    }
    CuAssertTrue(tc, 1);
}

static void psys_calloc_test(CuTest *tc)
{
    int i;
    char *alloc;
    
    alloc = psys_calloc(pchild, ALLOC_BYTES);
    CuAssertPtrNotNull(tc, alloc);

    for (i = 0; i < ALLOC_BYTES; i++) 
    {
        char *ptr = alloc + i;
        CuAssertTrue(tc, *ptr == '\0');
    }
}

static void psys_mem_pool_destroy_test(CuTest *tc)
{
    psys_mem_pool_destroy(pmain);
}

CuSuite *psys_memory_test(void)
{
    CuSuite *suite = CuSuiteNew("MemoryPool");

    SUITE_ADD_TEST(suite, psys_mem_pool_create_parent_test);
    SUITE_ADD_TEST(suite, psys_mem_pool_create_child_test);
    SUITE_ADD_TEST(suite, psys_malloc_test);
    SUITE_ADD_TEST(suite, psys_calloc_test);
    SUITE_ADD_TEST(suite, psys_mem_pool_destroy_test);

    return suite;
}
