/*
  +----------------------------------------------------------------------+
  | Portable System Library                                              |
  | Author: Sukchan Lee (acetcom@postech.ac.kr)                          |
  +----------------------------------------------------------------------+

  $Id: psys_tester.c 102 2004-03-26 15:36:02Z acetcom $
*/

#include "psys/common.h"
#include "psys/errno.h"
#include "psys/memory.h"

#include "CuTest.h"

CuSuite *psys_type_test(void);
CuSuite *psys_string_test(void);
CuSuite *psys_memory_test(void);

psys_mem_pool_t *test_mem_pool;

static const struct testlist {
    const char *testname;
    CuSuite *(*func)(void);
} tests[] = {
    {"type", psys_type_test},
    {"string", psys_string_test},
    {"memory", psys_memory_test},
    {"LastTest", NULL}
};

int main(int argc, char *argv[])
{
    CuSuiteList *alltests = NULL;
    CuString *output = CuStringNew();
    int i;

    psys_initialize();
    atexit(psys_terminate);

    psys_mem_pool_create(&test_mem_pool, NULL);

    CuInit(argc, argv);

    alltests = CuSuiteListNew("Portable System Library Tester");
    for (i = 0; tests[i].func != NULL; i++) 
    {
        CuSuiteListAdd(alltests, tests[i].func());
    }
    
    CuSuiteListRunWithSummary(alltests);
    i = CuSuiteListDetails(alltests, output);
    printf("%s\n", output->buffer);

    return i > 0 ? 1 : 0;
}

