/*
  +----------------------------------------------------------------------+
  | Portable System Library                                              |
  | Author: Sukchan Lee (acetcom@postech.ac.kr)                          |
  +----------------------------------------------------------------------+

  $Id: type.c 102 2004-03-26 15:36:02Z acetcom $
*/

#include "psys/common.h"
#include "psys/errno.h"

#include "CuTest.h"

static void psys_sizeof_test(CuTest *tc)
{
    CuAssertTrue(tc, sizeof(p_uint8_t) == 1);
    CuAssertTrue(tc, sizeof(p_uint16_t) == 2);
    CuAssertTrue(tc, sizeof(p_uint32_t) == 4);
    CuAssertTrue(tc, sizeof(p_uint64_t) == 8);
}

static void psys_maxmin_test(CuTest *tc)
{
    p_int8_t p_int8;
    p_int16_t p_int16;
    p_int32_t p_int32;
    p_int64_t p_int64;
    p_uint8_t p_uint8;
    p_uint16_t p_uint16;
    p_uint32_t p_uint32;
    p_uint64_t p_uint64;

    p_int8 = PSYS_INT8_MAX;
    CuAssertTrue(tc, ++p_int8 == PSYS_INT8_MIN);
    p_int16 = PSYS_INT16_MAX;
    CuAssertTrue(tc, ++p_int16 == PSYS_INT16_MIN);
    p_int32 = PSYS_INT32_MAX;
    CuAssertTrue(tc, ++p_int32 == PSYS_INT32_MIN);
    p_int64 = PSYS_INT64_MAX;
    CuAssertTrue(tc, ++p_int64 == PSYS_INT64_MIN);

    p_uint8 = PSYS_UINT8_MAX;
    CuAssertTrue(tc, ++p_uint8 == 0);
    p_uint16 = PSYS_UINT16_MAX;
    CuAssertTrue(tc, ++p_uint16 == 0);
    p_uint32 = PSYS_UINT32_MAX;
    CuAssertTrue(tc, ++p_uint32 == 0);
    p_uint64 = PSYS_UINT64_MAX;
    CuAssertTrue(tc, ++p_uint64 == 0);
}

static void psys_endian_test(CuTest *tc)
{
    p_uint16_t p_uint16_1, p_uint16_2;
    p_uint32_t p_uint32_1, p_uint32_2;
    p_uint64_t p_uint64_1, p_uint64_2;

    p_uint16_1 = 0x44af;
    p_uint16_2 = 0xaf44;
    CuAssertTrue(tc, PSYS_SWAP_ENDIAN16(p_uint16_1) == p_uint16_2);
    p_uint32_1 = PSYS_UINT32_C(0x02a7f109);
    p_uint32_2 = PSYS_UINT32_C(0x09f1a702);
    CuAssertTrue(tc, PSYS_SWAP_ENDIAN32(p_uint32_1) == p_uint32_2);
    p_uint64_1 = PSYS_UINT64_C(0x1d636b02300a7aa7);
    p_uint64_2 = PSYS_UINT64_C(0xa77a0a30026b631d);
    CuAssertTrue(tc, PSYS_SWAP_ENDIAN64(p_uint64_1) == p_uint64_2);
}

static void psys_ntoh_test(CuTest *tc)
{
    p_uint16_t p_uint16;
    p_uint32_t p_uint32;
    p_uint64_t p_uint64;

    p_uint16 = 0x44af;
    CuAssertTrue(tc, PSYS_NTOH16(PSYS_HTON16(p_uint16)) == p_uint16);
    p_uint32 = PSYS_UINT32_C(0x02a7f109);
    CuAssertTrue(tc, PSYS_NTOH32(PSYS_HTON32(p_uint32)) == p_uint32);
    p_uint64 = PSYS_UINT64_C(0x1d636b02300a7aa7);
    CuAssertTrue(tc, PSYS_NTOH64(PSYS_HTON64(p_uint64)) == p_uint64);
}

CuSuite *psys_type_test(void)
{
    CuSuite *suite = CuSuiteNew("Type");

    SUITE_ADD_TEST(suite, psys_sizeof_test);
    SUITE_ADD_TEST(suite, psys_maxmin_test);
    SUITE_ADD_TEST(suite, psys_endian_test);
    SUITE_ADD_TEST(suite, psys_ntoh_test);

    return suite;
}
